function rqaciGUI(varargin)

%rqaciGUI Compare to univariate time series
%
% function rqaciGUI(X,Y [,timeScale])
%
% Purpose:
% A simple MATLAB GUI that provides a convenient access to the 
% rqaci.m routine and the required plotting routines. In order to run
% the programme you have to have the CRPtoolbox installed as a prequesite
% for computing RPs.
%
% Input:
%	X = timeseries 1
%	Y = timeseries 2
%	ts = timeScale (used for plotting, optional)
%
% Parameter:
%	dim = embedding dimension
%	tau = embedding delay
%	eps = threshold
%	method = norm for RP
%	ws = window size
%	ss = step size
%	theiler = theiler window
%	lMin = minimal diagonal line length
%	vMin = minimal vertical line length
%	nSamples = number of bootstrap samples
%	alpha = alpha level for CI estimation
%
%	all parameters adjustable via GUI
%
% requires: CRPtoolbox, prctile (Statistics Toolbox)
%
% see also: opTool
%
% References: 
% S. Schinkel, N. Marwan, O. Dimigen & J. Kurths (2009): 
% "Confidence Bounds of recurrence-based complexity measures"
% Physics Letters A,  373(26), pp. 2245-2250


% Copyright (C) 2009 Stefan Schinkel, University of Potsdam
% http://www.agnld.uni-potsdam.de 
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

% $Log$


%% debug settings
debug = 0;
if debug,warning('on','all');warning('Matlab:DivideByZero','off');else warning('off','all');end

%% check number of input arguments
error(nargchk(2,3,nargin))

%% check number of out arguments
error(nargoutchk(0,0,nargout))

%% check input
varargin{4} = [];

%% initial I/O check
if ~isvector(varargin{1}); help(mfilename);error('Bad input format');else X = varargin{1};end
if ~isvector(varargin{2}); help(mfilename);error('Bad input format');else Y = varargin{2};end
if ~isempty(varargin{3}), timeScale = varargin{3};else timeScale = [];end

% of equal length
if length(X) ~= length(Y), help(mfilename);error('X & Y have to be of equal length');end

%% consistency and stuff
if isempty(timeScale);
	timeScale = 1:1:length(X);
else
	if length(timeScale) ~= length(X);
		warndlg('Timescale is not matching data. Not using it','Timescale Error','modal');
	end
end

%% some default parameters
dim = 2;tau = 1;epsilon = .1;
ws = round(length(X)*.1);
ss = round(length(X)*.01);
theiler = 1;minDiag = 2;minVert = 2;valRPNorm = 1;
valPlotSizeValue = 1;

% avail measures
measures = {'DET','L','LAM','TT'};

% avail RP norms
RPNorms = {'Fixed RR','Fixed Neighbours','Order Patterns','Euclidean Norm*','Maximimum Norm*','Minimum Norm*'};
RPNormKeys = {'rr','fan','op','eucl','max','min'};

%%%%%%%%%%%%%%%%%%%%%
%	Define GUI		%
%%%%%%%%%%%%%%%%%%%%%
screenSize = get(0,'Screensize');

figHandle = figure('Name','RQACI -- GUI',... 
	'Position',[50,screenSize(4)-650,800,600],... % top left corner 50x50 padding 
	'Color',[.801 .75 .688], ... 
	'Tag','mainFigure',...
	'Menubar','None');% choose 'Menubar','figure' to run into trouble

%%%%%%%%%%%%%%%%%%%%%
%	Layout GUI		%
%%%%%%%%%%%%%%%%%%%%%

set(figHandle,'visible','off')
axData = axes('Parent',figHandle,...
	'Position',[.05 .76 .9 .18],...
	'Tag', 'axData',...
	'NextPlot','replace',...
	'Box','on',...
	'Xtick',[],...
	'Ytick',[]);

axCI = axes('Parent',figHandle,...
	'Position',[.05 .275 .9 .45],...
	'Tag', 'axCI',...
	'NextPlot','replace',...
	'Box','on');


buttons = uibuttongroup('Position', [.05 .05 .55 .18],...
	'Title','RQA - parameters',...
	'visible','on');

	textDim = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.05 .71 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Dimension:',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Enter the embedding dimension.');

	textTau = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.05 .51 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Delay:',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Enter the embedding delay.');

	textWindowSize = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.04 .31 .16 .18],...
		'BackgroundColor', [0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Window size :',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','The size of the shifting window.');

	textStepSize = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.05 .11 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Step size :',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','The number of steps by which the window is shifted.');

	valDim = uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.225 .71 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(dim),...
		'Tag','valDim',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Insert the dimension (2-13)');	

	valTau = uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.225 .51 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(tau),...
		'Tag','valTau',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Insert the time delay');	
		
	valWindowSize= uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.225 .31 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(ws),...
		'Tag','valWindowSize',...		
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Insert the window size.');

	valStepSize = uicontrol('Parent',buttons,...
		'Units', 'normalized',...
		'Position', [.225 .11 .1 .18],...
		'BackgroundColor',[.801 .75 .688],...
		'Style','edit',... 
		'String',num2str(ss),...
		'Tag','valStepSize',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Insert the step size.');

	textEpsilon = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.35 .71 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Threshold:',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','The threshold used in computing the RP.');

	textTheiler = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.35 .51 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Theiler:',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','The size of the theiler window.');
		
	textMinDiag = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.35 .31 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','min. Diag.:',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','The minimal length of diagonal lines to be quantified');

	textMinVert = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.35 .11 .15 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','min. Vert.:',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','The minimal length of vertical lines to be quantified');

	valEpsilon = uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.55 .71 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(epsilon),...
		'Tag','valEpsilon',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Enter the size of the Theiler Window.');	
	
	valTheiler = uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.55 .51 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(theiler),...
		'Tag','valTheiler',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Enter the size of the Theiler Window.');	
		
	valMinDiag = uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.55 .31 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(minDiag),...
		'Tag','valMinDiag',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Enter the minimal length of diagonal lines.');	

	valMinVert = uicontrol('Parent',buttons,...
		'Units', 'normalized', ...
		'Position', [.55 .11 .1 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',... 
		'String',num2str(minVert),...
		'Tag','valMinVert',...
		'HorizontalAlignment','right',...
		'Visible','on',...
		'Tooltip','Enter the minimal length of vertical lines.');	
		
	textNorm = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.7 .75 .29 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Neighbourhood:',...
		'HorizontalAlignment','left',...
		'Visible','on',...
		'Tooltip',sprintf('Select how to define neighbourhood\n Asterisk denotes discouraged choices.'));

	valNorm = uicontrol('Parent',buttons,...
		'Units', 'normalized',...
		'Position', [.7 .575 .29 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','popupmenu',...
    	'String',RPNorms,...
		'Tag','valRPNorm',...
		'Value',valRPNorm,...
		'Tooltip',sprintf('Select how to define neighbourhood\n Asterisk denotes discouraged choices.'));

	textPlotSize = uicontrol('Parent',buttons, ...
		'Units', 'normalized', ...
		'Position', [.7 .3 .29 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','Size adjustment:',...
		'HorizontalAlignment','left',...
		'Visible','on',...
		'Tooltip','Adjust the size of the RP to compensate for embedding.');


	valPlotSize = uicontrol('Parent',buttons,...
		'Units', 'normalized',...
		'Position', [.7 .11 .29 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','popupmenu',...
    	'String',{'Use small RP','Use full RP'},...
		'Tag','valPlotSize',...
		'Value',valPlotSizeValue,...
		'Tooltip','Choose small for default CRPtool routines or full to adjust for embedding');		

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	measures, bootstrap params	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

buttons2 = uibuttongroup('Position', [.63 .05 .2 .18],...
	'Title','RQACI - parameters',...
	'visible','on');

	textMeasure = uicontrol('Parent',buttons2, ...
		'Units', 'normalized', ...
		'Position', [.1 .61 .5 .3],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String',sprintf('Measure:\n to plot'),...
		'HorizontalAlignment','left',...
		'Visible','on',...
		'Tooltip','Choose the RQA measure you would like to plot');

	valMeasure = uicontrol('Parent',buttons2,...
		'Units', 'normalized', ...
		'Position',[.65 .71 .3 .18],...
		'Style','popupmenu',...
    	'String',measures,...
		'Tag','valMeasure',...
		'Value',1,...
		'Callback',{@plotData,figHandle,timeScale});		

	textNBoot = uicontrol('Parent',buttons2, ...
		'Units', 'normalized', ...
		'Position', [.1 .31 .5 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','n Samples',...
		'HorizontalAlignment','left',...
		'Visible','on',...
		'Tooltip','The number of bootstrapped samples.');

	valNBoot = uicontrol('Parent',buttons2,...
		'Units', 'normalized', ...
		'Position',[.65 .31 .3 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',...
    	'String','500',...
		'HorizontalAlignment','right',...
		'Tag','valNBoot');		

	textAlpha = uicontrol('Parent',buttons2, ...
		'Units', 'normalized', ...
		'Position', [.1 .11 .5 .18],...
		'BackgroundColor',[0.7020 0.7020 0.7020], ...
		'Style','text',... 
		'String','alpha',...
		'HorizontalAlignment','left',...
		'Visible','on',...
		'Tooltip','Alpha level in percent. Determines the spread of CIs.');

	valAlpha = uicontrol('Parent',buttons2,...
		'Units', 'normalized', ...
		'Position',[.65 .11 .3 .18],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','edit',...
    	'String','5',...
		'HorizontalAlignment','right',...
		'Tag','valAlpha');		

		
%%%%%%%%%%%%%%%%%%%%%%%%%
%	Define buttons 		%
%%%%%%%%%%%%%%%%%%%%%%%%%

	buttonStore = uicontrol('Parent',figHandle, ...
		'Units', 'normalized', ...
		'Position', [.85 .17 .1 .06],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','pushbutton',... 
		'String','Print',...
		'Visible','on', ...
		'Tooltip','Save the current figure to file.',...
		'Callback',{@printCallback});

	buttonClose = uicontrol('Parent',figHandle, ...
		'Units', 'normalized', ...
		'Position', [.85 .11 .1 .06],...
		'BackgroundColor',[.801 .75 .688], ...
		'Style','pushbutton',... 
		'String','Close',...
		'Visible','on', ...
		'Tooltip','Close the GUI',...
		'Callback',{@closeCallback});

	buttonCompute = uicontrol('Parent',figHandle,...
		'Units', 'normalized',...
		'Position', [.85 .05 .1 .06],...
		'BackgroundColor',[.801 .75 .688],...
		'Style','pushbutton',... 
		'String','Compute',...
		'FontWeight','bold',...
		'Visible','on', ...
		'Tooltip','Start the computation',...
		'Callback',{@computeCallback,X,Y,RPNormKeys,measures,timeScale});

%%%%%%%%%%%%%%%%%%%%%
%	fancy stuff 	%
%%%%%%%%%%%%%%%%%%%%%

	textNLD = uicontrol('Parent',figHandle,...
		'Units', 'normalized', ...
		'Position',[.1 .01 .85 .03],...
		'Style','text',... 
		'BackgroundColor',[.801 .75 .688], ... 
		'Tag', 'NLD ',...
		'FontSize',9,...
		'String',sprintf('Stefan Schinkel - University of Potsdam (c) 2009'),...
		'Tooltip','Visit http://tocsy.agnld.uni-potsdam.de for details and other programmes',...
		'HorizontalAlignment','right');
	
	set(figHandle,'visible','on')

% plot source data
axes(axData);
plot(timeScale,X,'b');
hold on;
plot(timeScale,Y,'r');
set(axData,'Tag','axData');
axis tight

end %% main

%%%%%%%%%%%%%%%%%%%%%
% 	CALLBACKS		%
%%%%%%%%%%%%%%%%%%%%%
function computeCallback(source,eventdata,X,Y,RPNormKeys,measures,timeScale)

	figHandle = get(source,'Parent');
	
	%rqa params
	dim = str2double( get( findobj(figHandle,'Tag','valDim'),'String') );
	tau = str2double( get( findobj(figHandle,'Tag','valTau'),'String') );
	epsilon = str2double( get( findobj(figHandle,'Tag','valEpsilon'),'String') );
	ws = str2double( get( findobj(figHandle,'Tag','valWindowSize'),'String') );
	ss = str2double( get( findobj(figHandle,'Tag','valStepSize'),'String') );
	theiler = str2double( get( findobj(figHandle,'Tag','valTheiler'),'String') );
	minDiag = str2double( get( findobj(figHandle,'Tag','valMinDiag'),'String') );
	minVert = str2double( get( findobj(figHandle,'Tag','valMinVert'),'String') );
	plotSize = get( findobj(figHandle,'Tag','valPlotSize'),'Value');
	RPNorm = get( findobj(figHandle,'Tag','valRPNorm'),'Value');

	% rqaci params
	nBoot = str2double( get( findobj(figHandle,'Tag','valNBoot'),'String') );
	alpha = str2double( get( findobj(figHandle,'Tag','valAlpha'),'String') );

	% rqaci only makes sense if we window
	if ws == length(X);
		errordlg(sprintf('Please use windowing or\nuse rqabounds() instead.'),...
			'Suitability Error','modal')
		return
	end

	%% params aquired, compute stuff
	hWaitbar = waitbar(0,'Computing RQA measures ...','WindowStyle','modal');	

	%% for easier plotting later on	
	indeces =[];

	%switch small/full plot
	if plotSize == 1,
		for i=1:ss:length(X)-ws;
						
			rpX = crp(X(i:i+ws),dim,tau,epsilon,RPNormKeys{RPNorm},'silent');
			rpY = crp(Y(i:i+ws),dim,tau,epsilon,RPNormKeys{RPNorm},'silent');

			[valsX(i,:) cisX(i,:,:)] = rqaci(rpX,nBoot,alpha);
			[valsY(i,:) cisY(i,:,:)] = rqaci(rpY,nBoot,alpha);

			waitbar(i/length(X),hWaitbar);

			indeces(end+1) = i;
		end
	else
		%User info
		disp('INFO: Will use adjust plot to embedding params!');
		
		for i=1:ss:length(X)-ws-dim*tau;

			rpX = crp(X(i:i+ws+dim*tau),dim,tau,epsilon,RPNormKeys{RPNorm},'nogui','silent');
			rpY = crp(Y(i:i+ws+dim*tau),dim,tau,epsilon,RPNormKeys{RPNorm},'nogui','silent');

			[valsX(i,:) cisX(i,:,:)] = rqaci(rpX,nBoot,alpha);
			[valsY(i,:) cisY(i,:,:)] = rqaci(rpY,nBoot,alpha);

			waitbar(i/length(X),hWaitbar);

			indeces(end+1) = i;
			
		end
	end	

	%close waitbar
	close(hWaitbar)
	
	% merge data to struct
	data.cisX = cisX; data.cisY = cisY;
	data.indeces = indeces;
	
	% store in GUI
	set(figHandle,'UserData',data);

	%% manually trigger plotting callback 
	plotData(source,eventdata,figHandle,timeScale)

		
end % buttonComputeCallback

function plotData(source,eventdata,figHandle,timeScale)
	
	%get axis handles
	axData = findobj(figHandle,'Tag','axData');
	axCI = findobj(figHandle,'Tag','axCI');
	
	%% get params
	dim = str2double( get( findobj(figHandle,'Tag','valDim'),'String') );
	tau = str2double( get( findobj(figHandle,'Tag','valTau'),'String') );
	epsilon = str2double( get( findobj(figHandle,'Tag','valEpsilon'),'String') );
	ws = str2double( get( findobj(figHandle,'Tag','valWindowSize'),'String') );
	ss = str2double( get( findobj(figHandle,'Tag','valStepSize'),'String') );

	%% for fancying the title
	RPNorms = get( findobj(figHandle,'Tag','valRPNorm'),'String');
	RPNormValue = get( findobj(figHandle,'Tag','valRPNorm'),'Value');
	measures = get( findobj(figHandle,'Tag','valMeasure'),'String');
	measureSelected = get( findobj(figHandle,'Tag','valMeasure'),'Value');
	
	%see if sth has bee computed
	data = get(figHandle,'Userdata');

	if isempty(data),
		return
	else
		cisX = data.cisX;cisY = data.cisY;
		indeces = round(data.indeces);
	end
	
	%activate axCI	
	axes(axCI)
	cla(axCI)
	
	% plot data with given data indeces
	ciplot(cisX(indeces,measureSelected,1),cisX(indeces,measureSelected,2),...
		timeScale(indeces+round(ws/2)),'b')
	ciplot(cisY(indeces,measureSelected,1),cisY(indeces,measureSelected,2),...,
		timeScale(indeces+round(ws/2)),'r')	

	% fancy plot
	xlim([timeScale(1) timeScale(end)]);
	ylim('auto')
	set(axCI,'Tag','axCI');
	set(axCI,'XGrid','on');
	set(axCI,'YGrid','on');
	
	% and add the title
	title(axData,sprintf('%s for %s RP -- dim: %d tau: %d eps: %1.3f ws:%d ss:%d',...
		measures{measureSelected},RPNorms{RPNormValue},dim,tau,epsilon,ws,ss),...
		'Fontweight','bold');
	
end %plotData

	
%%%%%%%%%%%%%%%%%%%%%%%%%
% 	HELPER FUNCTIONS	%
%%%%%%%%%%%%%%%%%%%%%%%%%
function closeCallback(source,eventdata) 

	close( get(source,'Parent') )

end % buttonCloseCallback
function printCallback(source,eventdata) 
	
	% get access to figure
	figHandle = get(source,'Parent');
	
	% grab the plot axes for copying
	axData = findobj(figHandle,'Tag','axData');
	axCI = findobj(figHandle,'Tag','axCI');

	%new, hidden figure
	plotFigure = figure('visible','off')
	
	% copy data & CI axes
	copyobj(axData,plotFigure)
	copyobj(axCI,plotFigure)
	
	%serve printdlg
	printdlg(plotFigure);

end % buttonCloseCallback
